package com.nokia.common.gpload;

import com.nokia.common.gpload.entity.GploadResult;
import com.xxl.job.core.context.XxlJobHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class GploadUtil {
    public static final Pattern PATTERN = Pattern.compile("= (\\d+)");

    public static GploadResult gpload(String gploadCommand) {
        Process process = null;
        GploadResult result = new GploadResult();
        result.setTaskStatus(true);
        try {
            process = Runtime.getRuntime().exec(gploadCommand);
            try (BufferedReader inputReader = new BufferedReader(new InputStreamReader(process.getInputStream()));
                 BufferedReader errorReader = new BufferedReader(new InputStreamReader(process.getErrorStream()))) {
                String line;
                // 读取错误流
                List<String> error = new ArrayList<>();
                while ((line = errorReader.readLine()) != null) {
                    error.add(line);
                }
                if (!CollectionUtils.isEmpty(error)) {
                    log.info("gpload errorStream: " + error);
                    XxlJobHelper.log("gpload errorStream: " + error);
                    result.setTaskStatus(false);
                    result.setMessage(error.toString());
                }

                // 读取标准流
                List<String> input = new ArrayList<>();
                while ((line = inputReader.readLine()) != null) {
                    input.add(line);
                }
                if (!CollectionUtils.isEmpty(input)) {
                    log.info("gpload inputStream: " + input);
                    XxlJobHelper.log("gpload inputStream: " + input);
                    for (String s : input) {
                        if (s.contains("|rows Inserted")) {
                            result.setInsertedCount(getNum(s));
                        }
                        if (s.contains("|rows Updated")) {
                            result.setUpdatedCount(getNum(s));
                        }
                        if (s.contains("|data formatting errors")) {
                            result.setErrorCount(getNum(s));
                        }
                        if (s.contains("ERROR") || s.contains("gpload failed")) {
                            result.setTaskStatus(false);
                        }
                        if (s.contains("ERROR")) {
                            result.setMessage(result.getMessage() + "\n" + s);
                        }
                    }
                }
            }

            int code = process.waitFor();
            if (code != 0) {
                result.setTaskStatus(false);
            }
        } catch (IOException e) {
            log.error("gpload发生异常: {}", e.getMessage(), e);
            XxlJobHelper.log("gpload发生异常: {}", e.getMessage(), e);
            result.setTaskStatus(false);
            result.setMessage(e.getMessage());
        } catch (InterruptedException e) {
            log.error("gpload发生异常: {}", e.getMessage(), e);
            XxlJobHelper.log("gpload发生异常: {}", e.getMessage(), e);
            result.setTaskStatus(false);
            result.setMessage(e.getMessage());
            Thread.currentThread().interrupt();
        } finally {
            log.info("gpload的结果为: {}", result);
            XxlJobHelper.log("gpload的结果为: {}", result);
            if (process != null) {
                process.destroy();
            }
        }
        return result;
    }

    /**
     * 获取gpload统计条数
     */
    private static int getNum(String s) {
        Matcher matcher = PATTERN.matcher(s);
        if (matcher.find()) {
            return Integer.parseInt(matcher.group(1));
        }

        return 0;
    }
}
